#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

void showTime(time_t time, int blink=-1, uint8_t symbol=0);
bool changeTime(uint8_t symbol=0);

// The changeTimeCursorPosition holds what to change
// 0 = hour, 1 = minutes, 0 = seconds
int changeTimeCursorPosition;
// timeToChange is the time that should be changed.
// This must be flexibal because the real time and the alarm time must be changed
time_t timeToChange;


#define BLINK_SPEED 500
unsigned long blinkTimer;
bool blinkOn;


// This function shows the time
// time_t time is the time to display
// int blink is the number that should blink, -1 if no number should blink
// uint8_t symbol is the symbol it can print on the top right corner
void showTime(time_t time, int blink, uint8_t symbol) {
  String timestr;
  
  display.clearDisplay();
  display.setTextSize(4);
  display.setCursor(0, 5);
  
  if(blink == 0 && !blinkOn)
    timestr = "  ";
  else
    timestr = l0(hour(time));

  display.print(timestr);
  display.setCursor(40, 5);
  display.print(":");

  if(blink == 1 && !blinkOn)
    timestr = "  ";
  else
    timestr = l0(minute(time));

  display.setCursor(56, 5);
  display.print(timestr);
  
  if(blink == 2 && !blinkOn)
    timestr = "  ";
  else
    timestr = l0(second(time));
  
  display.setCursor(103, 18);
  display.setTextSize(2);
  display.print(timestr);
  
  display.setCursor(115, 5);
  display.setTextSize(1);
  display.write(symbol);
}

// This function converts an integer to a string with a leading 0
String l0(int n) {
  return ((n < 10) ? "0" : "") + String(n);
}


// The change time function does everything that lets the user change the time
bool changeTime(uint8_t symbol) {
  // At the biginning set the changeTimeCursorPosition = 0
  if(stateChanged == true) {
    changeTimeCursorPosition = 0;
  }

  // If the right button was clicked go to the next number to change
  if(buttonClicked[BTN_RIGHT]) {
    changeTimeCursorPosition++;
    blinkOn = false;
  }

  // finish changing and return true
  if(changeTimeCursorPosition == 3) {
    return true;
  }

  // if the button up or down was pressed change the time
  if(buttonClicked[BTN_DOWN])
    changeTimePart(-1);

  if(buttonClicked[BTN_UP])
    changeTimePart(1);

  // and finally show the time, here the blink argument is used
  showTime(timeToChange, changeTimeCursorPosition, symbol);

  // changing time is not finished yet -> return false
  return false;
}

// This changes the hour, minute or second of timeToChange and handles overflow
// v must be 1 or -1
void changeTimePart(int v) {
  TimeElements te;
  breakTime(timeToChange, te);
  
  if(changeTimeCursorPosition == 0) {
    te.Hour += v;

    if(te.Hour == 24)
      te.Hour = 0;

    if(te.Hour == 0xFF)
      te.Hour = 23;
  }
  
  if(changeTimeCursorPosition == 1) {
    te.Minute += v;

    if(te.Minute == 60)
      te.Minute = 0;

    if(te.Minute == 0xFF)
      te.Minute = 59;
  }
  
  if(changeTimeCursorPosition == 2) {
    te.Second += v;

    if(te.Second == 60)
      te.Second = 0;

    if(te.Second == 0xFF)
      te.Second = 59;
  }

  timeToChange = makeTime(te);
}
